@@ -45,9 +45,11 @@
 
 typedef struct {
   gboolean sandbox;
+  char *thumbnailer_name;
   GArray *fd_array;
   /* Input/output file paths outside the sandbox */
   char *infile;
+  char *infile_tmp; /* the host version of /tmp/gnome-desktop-file-to-thumbnail.* */
   char *outfile;
   char *outdir; /* outdir is outfile's parent dir, if it needs to be deleted */
   /* I/O file paths inside the sandbox */
@@ -498,9 +500,6 @@ static gboolean
 add_bwrap (GPtrArray   *array,
 	   ScriptExec  *script)
 {
-  char *fd_str;
-  int fd;
-
   g_return_val_if_fail (script->outdir != NULL, FALSE);
   g_return_val_if_fail (script->s_infile != NULL, FALSE);
 
@@ -532,25 +531,14 @@ add_bwrap (GPtrArray   *array,
   g_ptr_array_add (array, g_strdup (script->outdir));
   g_ptr_array_add (array, g_strdup ("/tmp"));
 
-  /* Open the infile so we can pass the fd through bwrap,
-   * we make sure to also re-use the original file's original
+  /* We make sure to also re-use the original file's original
    * extension in case it's useful for the thumbnailer to
    * identify the file type */
-  fd = open (script->infile, O_RDONLY | O_CLOEXEC);
-  if (fd == -1)
-    goto bail;
-  fd_str = g_strdup_printf ("%d", fd);
-  g_ptr_array_add (array, g_strdup ("--file"));
-  g_ptr_array_add (array, fd_str);
+  g_ptr_array_add (array, g_strdup ("--ro-bind"));
+  g_ptr_array_add (array, g_strdup (script->infile));
   g_ptr_array_add (array, g_strdup (script->s_infile));
 
-  g_array_append_val (script->fd_array, fd);
-
   return TRUE;
-
-bail:
-  g_ptr_array_set_size (array, 0);
-  return FALSE;
 }
 #endif /* HAVE_BWRAP */
 
@@ -569,6 +557,8 @@ expand_thumbnailing_cmd (const char  *cmd,
   if (!g_shell_parse_argv (cmd, NULL, &cmd_elems, error))
     return NULL;
 
+  script->thumbnailer_name = g_strdup (cmd_elems[0]);
+
   array = g_ptr_array_new_with_free_func (g_free);
 
 #ifdef HAVE_BWRAP
@@ -657,7 +647,16 @@ child_setup (gpointer user_data)
 static void
 script_exec_free (ScriptExec *exec)
 {
+  if (exec == NULL)
+    return;
+
+  g_free (exec->thumbnailer_name);
   g_free (exec->infile);
+  if (exec->infile_tmp)
+    {
+      g_unlink (exec->infile_tmp);
+      g_free (exec->infile_tmp);
+    }
   if (exec->outfile)
     {
       g_unlink (exec->outfile);
@@ -665,7 +664,11 @@ script_exec_free (ScriptExec *exec)
     }
   if (exec->outdir)
     {
-      g_rmdir (exec->outdir);
+      if (g_rmdir (exec->outdir) < 0)
+        {
+          g_warning ("Could not remove %s, thumbnailer %s left files in directory",
+                     exec->outdir, exec->thumbnailer_name);
+        }
       g_free (exec->outdir);
     }
   g_free (exec->s_infile);
@@ -684,7 +687,8 @@ clear_fd (gpointer data)
 }
 
 static ScriptExec *
-script_exec_new (const char *uri)
+script_exec_new (const char  *uri,
+		 GError     **error)
 {
   ScriptExec *exec;
   g_autoptr(GFile) file = NULL;
@@ -702,13 +706,18 @@ script_exec_new (const char *uri)
 
   exec->infile = g_file_get_path (file);
   if (!exec->infile)
-    goto bail;
+    {
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
+                   "Could not get path for URI '%s'", uri);
+      goto bail;
+    }
 
 #ifdef HAVE_BWRAP
   if (exec->sandbox)
     {
       char *tmpl;
       g_autofree char *ext = NULL;
+      g_autofree char *infile = NULL;
 
       exec->fd_array = g_array_new (FALSE, TRUE, sizeof (int));
       g_array_set_clear_func (exec->fd_array, clear_fd);
@@ -716,12 +725,18 @@ script_exec_new (const char *uri)
       tmpl = g_strdup ("/tmp/gnome-desktop-thumbnailer-XXXXXX");
       exec->outdir = g_mkdtemp (tmpl);
       if (!exec->outdir)
-        goto bail;
+        {
+          g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                               "Could not create temporary sandbox directory");
+          goto bail;
+        }
       exec->outfile = g_build_filename (exec->outdir, "gnome-desktop-thumbnailer.png", NULL);
-
       ext = get_extension (exec->infile);
-      exec->s_infile = g_strdup_printf ("/tmp/gnome-desktop-file-to-thumbnail.%s", ext);
-      exec->s_outfile = g_strdup ("/tmp/gnome-desktop-thumbnailer.png");
+      infile = g_strdup_printf ("gnome-desktop-file-to-thumbnail.%s", ext);
+      exec->infile_tmp = g_build_filename (exec->outdir, infile, NULL);
+
+      exec->s_infile = g_build_filename ("/tmp/", infile, NULL);
+      exec->s_outfile = g_build_filename ("/tmp/", "gnome-desktop-thumbnailer.png", NULL);
     }
   else
 #endif
@@ -729,7 +744,7 @@ script_exec_new (const char *uri)
       int fd;
       g_autofree char *tmpname = NULL;
 
-      fd = g_file_open_tmp (".gnome_desktop_thumbnail.XXXXXX", &tmpname, NULL);
+      fd = g_file_open_tmp (".gnome_desktop_thumbnail.XXXXXX", &tmpname, error);
       if (fd == -1)
         goto bail;
       close (fd);
@@ -756,7 +771,9 @@ gnome_desktop_thumbnail_script_exec (const char  *cmd,
   GBytes *image = NULL;
   ScriptExec *exec;
 
-  exec = script_exec_new (uri);
+  exec = script_exec_new (uri, error);
+  if (!exec)
+    goto out;
   expanded_script = expand_thumbnailing_cmd (cmd, exec, size, error);
   if (expanded_script == NULL)
     goto out;
