@@ -1066,13 +1066,20 @@ gnome_desktop_thumbnail_factory_generate_thumbnail (GnomeDesktopThumbnailFactory
   if (script)
     {
       GBytes *data;
+      GError *error = NULL;
 
-      data = gnome_desktop_thumbnail_script_exec (script, size, uri, NULL);
+      data = gnome_desktop_thumbnail_script_exec (script, size, uri, &error);
       if (data)
         {
           pixbuf = pixbuf_new_from_bytes (data, NULL);
           g_bytes_unref (data);
         }
+      else
+        {
+          g_debug ("Thumbnail script ('%s') failed for '%s': %s",
+                   script, uri, error ? error->message : "no details");
+          g_clear_error (&error);
+        }
     }
 
   g_free (script);
@@ -1109,7 +1116,7 @@ save_thumbnail (GdkPixbuf  *pixbuf,
     goto out;
   close (tmp_fd);
 
-  g_snprintf (mtime_str, 21, "%ld",  mtime);
+  g_snprintf (mtime_str, 21, "%" G_GINT64_FORMAT, (gint64) mtime);
   width = gdk_pixbuf_get_option (pixbuf, "tEXt::Thumb::Image::Width");
   height = gdk_pixbuf_get_option (pixbuf, "tEXt::Thumb::Image::Height");
 
